from typing import *


from sqlalchemy.engine.url import URL
from sqlalchemy import text, MetaData
from sqlalchemy.ext.asyncio import create_async_engine, AsyncEngine


_engine: Optional[AsyncEngine] = None
_is_started = False


async def create_db(engine_to_use, name: str):
    async with engine_to_use.connect() as conn:
        await conn.execute(text("COMMIT"))
        await conn.execute(text(f"CREATE DATABASE {name}"))


async def drop_db(engine_to_use, db_name: str):
    async with engine_to_use.connect() as conn:
        await conn.execute(text("COMMIT"))
        await conn.execute(text(f"DROP DATABASE IF EXISTS {db_name}"))


def create_test_database_url(url_to_modify: URL) -> URL:
    """
    Returns a modified URL where the database name is suffixed with _test
    :return:
    """
    return url_to_modify.set(database=url_to_modify.database + "_test")


async def init_async_test_db(db_url: URL, metadata: MetaData) -> AsyncEngine:
    """
    Connects to the configured database temporarily, issues CREATE commands to instantiate a test database,
    disposes the engine and replaces the global engine with the test one.

    :return: Reference to the instantiated test engine.
    """
    global _engine
    global _is_started

    temp_engine = create_async_engine(db_url)
    test_db_url = create_test_database_url(db_url)

    await drop_db(temp_engine, test_db_url.database)
    await create_db(temp_engine, test_db_url.database)

    _engine = create_async_engine(test_db_url)

    async with _engine.begin() as conn:
        await conn.run_sync(metadata.create_all)

    _is_started = True

    return _engine


async def destroy_async_test_db(db_url):
    """
    If the module's engine is a test engine, it will dispose it.
    Then, connects to the given db_url as "main" database and drops the test database.

    :param db_url:
    :return:
    """
    global _engine
    global _is_started

    if _is_started is True:
        await _engine.dispose()

        temp_engine = create_async_engine(db_url)
        test_db_url = create_test_database_url(db_url)

        await drop_db(temp_engine, test_db_url.database)
        _is_started = False

    return



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['spawndb']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy[asyncio]>=1.4.31,<2.0.0']

setup_kwargs = {
    'name': 'spawndb',
    'version': '0.1.13',
    'description': '',
    'long_description': "# TestDB\n\nThis is a simple library that helps in the creation of test databases for Python\nprojects that use SQLAlchemy.\n\nIt's aim it's to seamlessly handle creation of a separate test database, including\nthe creation of all schema objects and finally, handling the destruction of the test\ndatabase when it's no longer needed.\n\n# Usage\nTo create a test database, use the init_test_db function.\n\nThis function expects two arguments:  \n   - *database_url*: sqlalchemy.engine.URL  \n   - *metadata*: sqlalchemy.schema.MetaData\n\nIt will return an instantiated Engine for the test database which you can use for \nyour database logics.\n\n```python\n# Sample usage for Pytest\nfrom testdb import init_test_db, destroy_test_db\n\ndef my_cool_test():\n    db_engine = init_test_db(database_url, sqla_metadata)\n    \n    try:\n        # your stuff goes here\n    \n    finally:\n        destroy_test_db(database_url)\n```\n\n",
    'author': 'Pablo Prieto',
    'author_email': 'pabloprieto@live.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
